# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from collections import deque

from _sadm.utils.cmd import call

__all__ = ['check']

def check(env, action = None):
	diff = deque()
	dn = env.dist()
	l = ('install', 'remove', 'prune')
	if action is not None and action in l:
		l = (action,)
	doCommon = env.settings.getboolean('os.pkg', 'common.enable', fallback = True)
	commonDisable = []
	if not doCommon:
		for x in l:
			commonDisable.append("%s.%s" % (dn, x))
	for opt in env.settings['os.pkg']:
		if opt in commonDisable:
			continue
		if opt in l:
			_check(env, opt, diff)
			continue
		if opt.startswith(dn + '.'):
			for n in l:
				if opt.endswith('.' + n):
					_check(env, opt, diff)
	return diff

def _check(env, opt, diff):
	l = []
	act = None

	if opt == 'remove' or opt.endswith('.remove'):
		act = 'remove'
		l = env.settings.getlist('os.pkg', opt)

	elif opt == 'install' or opt.endswith('.install'):
		act = 'install'
		l = env.settings.getlist('os.pkg', opt)

	elif opt == 'prune' or opt.endswith('.prune'):
		act = 'prune'
		l = env.settings.getlist('os.pkg', opt)

	for pkg in l:

		if act == 'remove':
			rc = call("/usr/bin/dpkg-query -s %s >/dev/null 2>/dev/null" % pkg)
			if rc == 0:
				diff.append((opt, pkg))
			elif rc > 1:
				raise env.error("os.pkg debian dpkg-query -s failed (rc:%d)" % rc)

		elif act == 'install':
			rc = call("/usr/bin/dpkg-query -s %s >/dev/null 2>/dev/null" % pkg)
			if rc == 1:
				diff.append((opt, pkg))
			elif rc > 1:
				raise env.error("os.pkg debian dpkg-query -s failed (rc:%d)" % rc)

		elif act == 'prune':
			rc = call("/usr/bin/dpkg-query -s %s >/dev/null 2>/dev/null" % pkg)
			if rc == 0:
				diff.append((opt, pkg))
			elif rc > 1:
				raise env.error("os.pkg debian dpkg-query -s failed (rc:%d)" % rc)



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from _sadm.utils.cmd import callCheck as call

from .check import check

__all__ = ['deploy']

def deploy(env):
	env.log('os user')
	for diff in check(env):
		typ = diff[0]
		if typ == 'group':
			_addGroup(env, diff[1], diff[2])
		elif typ == 'user':
			_addUser(env, diff[1], diff[2])
		else:
			raise RuntimeError("unknown os.user check type: %s" % typ)

def _addGroup(env, group, gid):
	cmd = ['groupadd', '-g', str(gid), group]
	call(cmd)
	env.log("%d %s group created" % (gid, group))

def _addUser(env, user, uid):
	cmd = ['useradd', '-m', '-U', '-u', str(uid)]

	fullname = env.settings.get("os.user.%s" % user, 'fullname', fallback = '').strip()
	if fullname != '':
		cmd.append('-c')
		cmd.append(fullname)

	shell = env.settings.get("os.user.%s" % user, 'shell', fallback = '/bin/bash').strip()
	cmd.append('-s')
	cmd.append(shell)

	cmd.append(user)
	call(cmd)
	env.log("%d %s user created" % (uid, user))



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

try:
	from _sadm._version import version as _version
except ImportError:
	_version = 'master'

try:
	from _sadm._version_build import version_build as _version_build
except ImportError:
	_version_build = 'devel'

from _sadm import version

def test_get():
	assert version.get() == _version

def test_build():
	assert version.build() == _version_build

def test_string():
	s = "version %s (build %s)" % (_version, _version_build)
	assert version.string() == '%(prog)s ' + s



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

__all__ = ['configure']

def configure(env, cfg):
	env.settings.merge(cfg, 'os', (
		'hostname',
		'hostname.file',
		'users.home.dir',
	))



#!/usr/bin/env python3
# coding: utf-8

# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

# https://packaging.python.org/guides/distributing-packages-using-setuptools/

from datetime import datetime
from os import makedirs, path, unlink
from setuptools import setup, find_packages
from sys import argv

def _clean():
	for n in ('_version.py', '_version_build.py'):
		fn = path.join('_sadm', n)
		if path.isfile(fn):
			unlink(fn)

def _buildInfo():
	now = datetime.utcnow()
	with open('./_sadm/_version_build.py', 'w') as fh:
		fh.write('# auto-generated file\n')
		fh.write("version_build = '%s'\n" % now.strftime('%y%m%d.%H%M%S'))

def main():

	try:
		cmd = argv[1]
	except IndexError:
		cmd = 'none'

	with open('requirements.txt', 'r') as fh:
		deps = fh.read().splitlines()

	_buildInfo()

	setup(
		author = 'Jeremías Casteglione',
		author_email = 'jrmsdev@gmail.com',
		python_requires = '~=3.6',
		setup_requires = ['wheel>=0.33', 'setuptools_scm>=3.3'],
		install_requires = deps,
		use_scm_version = {
			'write_to': '_sadm/_version.py',
			'fallback_version': '0.0',
		},
		py_modules = ['sadm'],
		packages = find_packages(),
		include_package_data = True,
	)

	if cmd == 'clean':
		_clean()

if __name__ == '__main__':
	main()



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

import sys
from os import path, makedirs, unlink, chmod
from shutil import rmtree, unpack_archive
from subprocess import call

from _sadm import log, cfg, deploy

def loadenv(filename): # pragma: no cover
	envfn = path.realpath(filename)
	log.msg("%s: load" % envfn)
	rc = _check(envfn)
	if rc != 0:
		return rc
	if path.isfile(envfn + '.asc'):
		rc = _verify(envfn)
		if rc != 0:
			return rc
	rc = call("sha256sum -c %s" % envfn, shell = True)
	if rc != 0:
		log.error('env checksum failed!')
		return rc
	_importenv(envfn)
	return 0

def _check(envfn):
	env = path.basename(envfn)[:-4]
	config = cfg.new(deploy.cfgfile)
	if not env in config.listEnvs('deploy'):
		log.error("%s env not found" % env)
		return 1
	return 0

def _verify(envfn):
	rc = call("gpg --no-tty --no --verify %s.asc %s 2>/dev/null" % (envfn, envfn),
		shell = True)
	if rc == 0:
		log.msg("%s: OK" % envfn)
	else:
		log.error('env signature verify failed!')
	return rc

def _importenv(envfn):
	srcfn = envfn[:-4]
	rootdir = path.dirname(path.dirname(envfn))
	deploydir = path.join(rootdir, 'deploy')
	envdir = path.join(deploydir, path.basename(srcfn))
	if path.isdir(envdir): # pragma: no cover
		rmtree(envdir)
	makedirs(envdir)
	chmod(envdir, 0o0700)
	log.msg("%s.zip: unpack" % srcfn)
	unpack_archive(srcfn + '.zip', extract_dir = envdir, format = 'zip')



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from _sadm import log, env, deploy

def run(envname):
	log.debug("run %s" % envname)
	rc, _ = env.run('deploy', envname, 'deploy', cfgfile = deploy.cfgfile)
	return rc



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from contextlib import contextmanager
from os import path, unlink
from time import time

from _sadm import log, cfg, asset, build, dist
from _sadm.configure import plugins, getPlugin
from _sadm.env import action as envAction
from _sadm.env.profile import Profile
from _sadm.env.session import Session
from _sadm.env.settings import Settings
from _sadm.errors import Error, EnvError

__all__ = ['Env', 'run']

class Env(object):
	_name = None
	_cfgfile = None
	_profile = None
	_profName = None
	_logtag = None
	_run = None
	_lockfn = None
	_err = None
	config = None
	assets = None
	settings = None
	session = None
	build = None

	def __init__(self, profile, name, config = None):
		if config is None:
			config = cfg.new()
		self.config = config
		if profile == 'default':
			profile = self.config.get('default', 'profile')
		if name == 'default':
			name = self.config.get('default', 'env')
		self._name = name
		self._profile = Profile(profile, self.config)
		self._profName = self._profile.name()
		self._logtag = ''
		self._run = {}
		self.settings = Settings()
		self.session = Session()
		self._load()
		self._check()

	def _load(self, fn = None, pdir = None):
		self.debug('env load')
		# env self.config.ini
		opt = "env.%s" % self._name
		if fn is None:
			fn = self.config.get(self._profName, opt, fallback = None)
			if fn is None:
				raise self.error("'%s/%s' env not found" % (self._profName, self._name))
		fn = path.normpath(fn.strip())
		if fn == '.':
			raise self.error('config file not set')
		self._cfgfile = fn
		# profile dir
		if pdir is None:
			pdir = self.config.get(self._profName, 'dir')
		pdir = path.normpath(pdir.strip())
		if not path.exists(pdir):
			raise self.error("%s directory not found" % pdir)
		if not path.isdir(pdir):
			raise self.error("%s is not a directory" % pdir)
		# env assets
		_rootdir = path.realpath(pdir)
		self.assets = asset.Manager(_rootdir)
		self.debug("assets %s" % _rootdir)
		# env builddir
		_builddir = path.join(path.dirname(_rootdir), 'build',
			self._profName, self._name)
		self.build = build.Manager(_builddir)
		self.debug("builddir %s" % self.build.rootdir())

	def _check(self):
		# check env dist setting
		if self.dist() not in dist.SUPPORTED.keys():
			raise self.error("unsupported dist %s" % self.dist())

	def configure(self, cfgfile = None):
		self.debug('session start')
		self.session.start()
		plugins.configure(self, cfgfile = cfgfile)

	def name(self):
		return self._name

	def profile(self):
		return self._profName

	def cfgfile(self):
		return self._cfgfile

	def _log(self, func, msg):
		func("%s/%s%s %s" % (self._profName, self._name, self._logtag, msg))

	def log(self, msg):
		self._log(log.msg, msg)

	def info(self, msg):
		self._log(log.info, msg)

	def warn(self, msg):
		self._log(log.warn, msg)

	def debug(self, msg):
		tag = "%s/%s%s" % (self._profName, self._name, self._logtag)
		log.debug("%s" % msg, depth = 4, tag = tag)

	def error(self, msg):
		self._err = EnvError(msg)
		self._log(log.error, msg)
		return self._err

	def getError(self):
		return self._err

	def start(self, action, msg = ''):
		if self._run.get(action, None) is not None:
			raise self.error("%s action already started" % action)
		prev = self._logtag
		self._logtag = " %s" % action
		sep = ''
		if msg != '':
			sep = ' '
		self.info("start%s%s" % (sep, msg))
		self._run[action] = {'tag.prev': prev, 'start': time()}

	def end(self, action):
		if self._run.get(action, None) is None:
			raise self.error("%s action was not started" % action)
		self._run[action]['end'] = time()
		self.info("end (%fs)" % (self._run[action]['end'] - self._run[action]['start']))
		self._logtag = self._run[action]['tag.prev']

	def report(self, action, startTime = None):
		actno = 0
		noend = []
		for n, d in self._run.items():
			actno += 1
			if d.get('end', None) is None:
				noend.append(n)
		took = ''
		if startTime is not None:
			took = " in %f seconds" % (time() - startTime)
		self.log("%s %d/%d actions%s" % (action, (actno - len(noend)), actno, took))
		if len(noend) > 0:
			raise self.error("not finished action(s): %s" % ','.join(noend))

	def plugins(self, action, revert = False):
		for p in self.settings.plugins(revert = revert):
			yield getPlugin(p, action)

	def dist(self):
		return self.settings.get('sadm', 'env.dist', fallback = 'debian')

	@contextmanager
	def lock(self):
		try:
			yield _lock(self)
		finally:
			_unlock(self)

def _lock(env):
	fn = env.assets.rootdir(path.dirname(env.cfgfile()), '.lock')
	env.debug("lock %s" % fn)
	envdir = path.dirname(fn)
	try:
		fh = open(fn, 'x')
	except FileExistsError:
		raise env.error("lock file exists: %s" % fn)
	except FileNotFoundError:
		raise env.error("env dir %s does not exists?" % envdir)
	fh.write('1')
	fh.flush()
	fh.close()
	env._lockfn = fn
	return env

def _unlock(env):
	if env._lockfn is None:
		env.debug('unlock env: not locked')
	else:
		env.debug("unlock %s" % env._lockfn)
		try:
			unlink(env._lockfn)
		except FileNotFoundError:
			raise env.error("unlock file not found: %s" % env._lockfn)
		finally:
			env._lockfn = None

def run(profile, env, action, cfgfile = None):
	err = None
	try:
		e = Env(profile, env, cfg.new(cfgfile = cfgfile))
		envAction.run(e, action)
	except EnvError as err:
		return (1, err)
	except Error as err:
		log.error("%s" % err)
		return (2, err)
	return (0, err)



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from os.path import realpath, dirname, join, normpath, sep

__all__ = ['fpath', 'fopen']

_srcdir = realpath(dirname(__file__))

def root():
	return _srcdir

def fpath(*parts):
	n = join(*parts)
	n = normpath(n)
	while n.startswith(sep):
		n = n.replace(sep, '', 1)
	return join(_srcdir, n)

def fopen(*parts):
	return open(fpath(*parts), 'r', encoding = 'utf-8')



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from hashlib import sha256
from subprocess import call

from _sadm import libdir
from _sadm.deploy import extractor
from _sadm.errors import BuildError
from _sadm.utils import path, builddir

__all__ = ['pre_build', 'build', 'post_build']

#
# pre build
#

def pre_build(env):
	builddir.lock(env)
	_writeSettings(env)

def _writeSettings(env):
	env.log('configure.ini')
	fn = None
	with builddir.create(env, 'configure.ini', meta = True) as fh:
		env.settings.write(fh)
		fn = fh.name
	h = sha256()
	with open(fn, 'rb') as fh:
		h.update(fh.read())
	env.session.set('sadm.configure.checksum', h.hexdigest())

#
# build
#

def build(env):
	if env.session.get('sadm.listen.enable', False):
		env.log('enable sadm-listen')
		_buildListen(env)

def _buildListen(env):
	cfgfiles = (
		(libdir.fpath('listen', 'uwsgi.ini'),
			path.join(path.sep, 'etc', 'opt', 'sadm', 'uwsgi.ini')),
		(libdir.fpath('listen', 'uwsgi.service'),
			path.join(path.sep, 'etc', 'systemd', 'system', 'sadm-listen.service')),
	)
	for srcfn, dstfn in cfgfiles:
		env.log("create %s" % dstfn)
		with open(srcfn, 'r') as src:
			with builddir.create(env, dstfn) as dst:
				dst.write(src.read())
#
# post build
#

def post_build(env):
	_saveSession(env)
	_signBuild(env)
	extractor.gen(env)
	builddir.unlock(env)

def _saveSession(env):
	env.debug('session.json')
	with builddir.create(env, 'session.json', meta = True) as fh:
		env.session.dump(fh)

def _signBuild(env):
	sid = env.config.get(env.profile(), 'build.sign', fallback = '')
	sid = sid.strip()
	if sid != '':
		env.log("sign id %s" % sid)
		fn = path.normpath(env.build.rootdir()) + '.env'
		env.log("sign env %s" % fn)
		rc = call("gpg --no-tty --yes -u '%s' -sba %s" % (sid, fn), shell = True)
		if rc != 0:
			raise BuildError('build sign using gpg failed')



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from _sadm.configure import pluginsList

def test_configure_testing(testing_plugin):
	p = testing_plugin('testing', ns = '_sadmtest')
	assert p.configure()

def test_all_configure(testing_plugin):
	for n in pluginsList():
		if n in ('testing', 'sadm', 'sadmenv'):
			continue
		print('-- configure plugin:', n)
		p = testing_plugin(n)
		assert p.configure(), "%s plugin.configure failed" % n



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

def configure(env, cfg):
	if 'testing' in cfg:
		for opt in cfg['testing']:
			env.settings['testing'][opt] = cfg['testing'][opt]



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from collections import deque, namedtuple
from importlib import import_module
from os import path

from _sadm import log
from _sadm.errors import PluginError

__all__ = ['register', 'getPlugin', 'pluginList']

_reg = {}
_order = deque()

Plugin = namedtuple('Plugin', ('name', 'fullname', 'config', 'meta', 'mod'))

def register(name, filename):
	if name.startswith('_sadmtest.'):
		n = name.replace('_sadmtest.plugin.', '')
	else:
		n = name.replace('_sadm.plugin.', '')
	if _reg.get(n, None) is not None:
		raise RuntimeError("plugin %s already registered" % name)
	srcdir = path.realpath(path.dirname(filename))
	cfgfn = path.join(srcdir, 'config.ini')
	metafn = path.join(srcdir, 'meta.json')
	_reg[n] = {
		'name': name,
		'srcdir': srcdir,
		'config': cfgfn,
		'meta': metafn,
	}
	_order.append(n)

def pluginsList(revert = False):
	_l = list
	if revert:
		_l = reversed
	for p in _l(_order):
		yield p

def getPlugin(name, action):
	p = _reg.get(name, None)
	if p is None:
		raise PluginError("%s plugin not found" % name)
	pkg = p['name']
	try:
		mod = import_module("%s.%s" % (pkg, action))
	except ImportError as err:
		log.debug("%s: %s" % (type(err), err))
		raise PluginError("%s.%s: %s" % (name, action, err))
	except Exception as err:
		raise PluginError("%s %s: %s" % (name, action, err))
	return Plugin(
		name = name,
		fullname = pkg,
		config = p['config'],
		meta = p['meta'],
		mod = mod,
	)



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from _sadm.web.app import wapp

# load views
import _sadm.web.errors
import _sadm.web.view.home
import _sadm.web.view.profile
import _sadm.web.view.syslog
import _sadm.web.view.about

def start(host, port, debug):
	wapp.run(host = host, port = port, reloader = debug,
		quiet = not debug, debug = debug)



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from _sadm.utils import systemd
from _sadm.utils.cmd import callCheck

__all__ = ['deploy']

def deploy(env):
	env.log('newaliases')
	callCheck('newaliases')
	systemd.restart('postfix')



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

import json
from bottle import request

from _sadm import log
from _sadm.listen.errors import error
from _sadm.listen.wapp import wapp
from _sadm.listen.webhook.repo.vcs.git import GitRepo

__all__ = ['exech']

_taskman = {
	'webhook.repo.git': GitRepo(),
}

@wapp.route('/_/exec/<task>/<action>', 'POST')
def exech(task, action):
	log.debug("exec handle: %s %s" % (task, action))
	taskman = _taskman.get(task, None)
	if taskman is None:
		raise error(500, "listen.exec task %s: no manager" % task)
	try:
		args = json.load(request.body)
		taskman.hook(action, args)
	except Exception as err:
		raise error(500, "%s" % err)
	return 'OK\n'



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from datetime import datetime
from _sadm import log, version

_viewreg = {}

def data(view):
	if _viewreg.get(view, False):
		raise RuntimeError("%s view already registered" % view)
	_viewreg[view] = True
	def wrapper(func):
		def decorator(*args, **kwargs):
			_start = datetime.timestamp(datetime.now())
			return _tplData(view, func(*args, **kwargs), _start)
		return decorator
	return wrapper

def _tplData(view, d, _start):
	log.debug("view data %s" % view)
	d['view'] = view
	d['version'] = version.get()
	d['now'] = datetime.strftime(datetime.now(), '%Y-%m-%d %H:%M:%S')
	d['took'] = "%.5f" % (datetime.timestamp(datetime.now()) - _start)
	return d



#!/usr/bin/env python3

# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

# sadm.env self extractor
# https://pypi.org/project/sadm/

import sys

from base64 import b64decode
from os import path, makedirs, chmod, unlink, getuid
from shutil import rmtree
from subprocess import call

_cargo = {}
_vars = {}

def main():
	env = _vars['env']
	rootdir = _vars['rootdir']
	dstdir = path.join(rootdir, 'env')
	extract(dstdir)
	envfn = path.join(dstdir, "%s.env" % env)
	envcmd = path.join(rootdir, 'bin', 'sadm')
	rc = call("%s --log info import %s" % (envcmd, envfn), shell = True)
	if rc != 0:
		return rc
	sudo = ''
	if getuid() != 0:
		sudo = '/usr/bin/sudo -n '
	rc = call("%s%s --log info --env %s deploy" % (sudo, envcmd, env), shell = True)
	if rc != 0:
		return rc
	return 0

def extract(dstdir):
	makedirs(dstdir, exist_ok = True)
	chmod(dstdir, 0o0700)
	for fn, data in _cargo.items():
		fn = path.join(dstdir, fn)
		if path.isfile(fn):
			unlink(fn)
		with open(fn, 'wb') as fh:
			fh.write(b64decode(data.encode()))

if __name__ == '__main__':
	sys.exit(main())



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

# register plugins (order really matters)

import _sadm.plugin.sadm
import _sadm.plugin.sadmenv

# os stuff should be done before sync assets are deployed

import _sadm.plugin.os
import _sadm.plugin.os.pkg
import _sadm.plugin.os.user

# deploy assets after all os stuff was done but before services/apps setup
# so services/apps deploy can use built (already deployed) assets
import _sadm.plugin.sync

# network plugins

import _sadm.plugin.network.iptables
import _sadm.plugin.network.fail2ban

# vcs clone should happen between network and service stuff
import _sadm.plugin.vcs.clone

# docker stuff
import _sadm.plugin.docker

# service plugins

import _sadm.plugin.service.postfix

import _sadm.plugin.service.munin
import _sadm.plugin.service.munin_node

# services that depend on apache should go before it
# so apache is restarted after all deps were setup
import _sadm.plugin.service.apache



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from configparser import ConfigParser
from os import path
from pytest import raises

from _sadm.env.settings import Settings
from _sadm.errors import SettingsError

def test_settings(testing_env):
	env = testing_env()
	assert isinstance(env.settings, Settings)
	assert isinstance(env.settings, ConfigParser)

def test_plugins(testing_settings):
	s = testing_settings()
	assert sorted(s.plugins('configure')) == ['os', 'sadm', 'sync', 'testing']

def test_parsing_error(testing_settings):
	with raises(SettingsError, match = 'Source contains parsing errors: '):
		testing_settings(cfgfile = 'config-parsing-error.ini')

def test_getlist_fallback(testing_settings):
	s = testing_settings()
	l = s.getlist('testing', 'test.notset.list', fallback = None)
	assert l is None
	with raises(SettingsError,
		match = 'No option \'test.notset.list\' in section: \'testing\''):
		s.getlist('testing', 'test.notset.list')

def test_getlist(testing_settings):
	s = testing_settings(cfgfile = 'config-settings-getlist.ini')
	l = s.getlist('testing', 'test.list.empty')
	assert isinstance(l, tuple)
	assert len(l) == 0

def test_getlist_comma(testing_settings):
	s = testing_settings(cfgfile = 'config-settings-getlist.ini')
	l = s.getlist('testing', 'test.list.comma')
	assert l == ('v0', 'v1', 'v2', 'v3')

def test_getlist_order(testing_settings):
	s = testing_settings(cfgfile = 'config-settings-getlist.ini')
	l = s.getlist('testing', 'test.list.order')
	assert l == ('v1', 'v3', 'v2', 'v0')

def test_getlist_lines(testing_settings):
	s = testing_settings(cfgfile = 'config-settings-getlist.ini')
	l = s.getlist('testing', 'test.list.lines')
	assert l == ('v0', 'v1', 'v2', 'v3')

def test_getlist_combined(testing_settings):
	s = testing_settings(cfgfile = 'config-settings-getlist.ini')
	l = s.getlist('testing', 'test.list.combined')
	assert l == ('v0', 'v1', 'v2', 'v3', 'v4', 'v5', 'v6')

def test_merge(testing_settings):
	cfgfn = path.join('tdata', 'testing', 'config-merge.ini')
	s = testing_settings()
	assert sorted(s.sections()) == ['os', 'sadm', 'sync', 'testing']
	assert s.options('testing') == []
	cfg = Settings()
	with open(cfgfn, 'r') as fh:
		cfg.read_file(fh)
	s.merge(cfg, 'nosection', ('opt0', 'opt1'))
	assert sorted(s.sections()) == ['os', 'sadm', 'sync', 'testing']
	s.merge(cfg, 'newsection', ('opt0', 'opt1'))
	assert sorted(s.sections()) == ['newsection', 'os', 'sadm', 'sync', 'testing']
	s.merge(cfg, 'testing', ('noopt0', 'noopt1', 'noopt2'))
	assert s.options('testing') == []
	s.merge(cfg, 'testing', ('opt0', 'opt1', 'opt2'))
	assert sorted(s.options('testing')) == ['opt0', 'opt1', 'opt2']

def test_merge_filter(testing_settings):
	def optfilter(opt):
		if opt == 'opt1' or opt == 'opt3':
			return opt
		return None
	cfgfn = path.join('tdata', 'testing', 'config-merge.ini')
	s = testing_settings()
	cfg = Settings()
	with open(cfgfn, 'r') as fh:
		cfg.read_file(fh)
	assert s.options('testing') == []
	s.merge(cfg, 'testing', optfilter)
	assert sorted(s.options('testing')) == ['opt1', 'opt3']



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

class Error(Exception):
	typ = None
	msg = None

	def __init__(self, msg):
		self.msg = msg

	def __str__(self):
		return "%s: %s" % (self.typ, self.msg)

	def __repr__(self):
		return "<sadm.%s: %s>" % (self.typ, self.msg)

	def __eq__(self, err):
		return self.typ == err.typ

	def __hash__(self):
		return hash(self.typ)

class ProfileError(Error):
	typ = 'ProfileError'

class EnvError(Error):
	typ = 'EnvError'

class SettingsError(Error):
	typ = 'SettingsError'

class SessionError(Error):
	typ = 'SessionError'

class AssetError(Error):
	typ = 'AssetError'

class AssetNotFoundError(Error):
	typ = 'AssetNotFoundError'

class BuildError(Error):
	typ = 'BuildError'

class CommandError(Error):
	typ = 'CommandError'

class PluginError(Error):
	typ = 'PluginError'

class PluginScriptNotFound(PluginError):
	typ = 'PluginScriptNotFound'

class PluginScriptNoExec(PluginError):
	typ = 'PluginScriptNoExec'

class PluginScriptTimeout(PluginError):
	typ = 'PluginScriptTimeout'

class ServiceError(PluginError):
	typ = 'ServiceError'



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

# coding: utf-8

from _sadm import version
from _sadm.cmd import build as build_cmd
from _sadm.cmd import deploy as deploy_cmd
from _sadm.cmd import web as web_cmd

__all__ = ['build', 'deploy', 'web']
__license__ = 'BSD'
__version__ = version.get()
__author__ = 'Jeremías Casteglione <jrmsdev@gmail.com>'

build = build_cmd.main
deploy = deploy_cmd.main
web = web_cmd.main



# Copyright (c) Jeremías Casteglione <jrmsdev@gmail.com>
# See LICENSE file.

from os import environ

from _sadm.utils.cmd import call, callCheck

from .check import check

__all__ = ['deploy']

_cmdenv = environ.copy()
_cmdenv['DEBIAN_FRONTEND'] = 'noninteractive'

def deploy(env):
	if env.settings.getboolean('os.pkg', 'update', fallback = False):
		env.log('update')
		_update()
	for diff in check(env, action = 'remove'):
		opt, pkg = diff
		env.log("%s %s" % (opt, pkg))
		_remove(pkg)
	for diff in check(env, action = 'install'):
		opt, pkg = diff
		env.log("%s %s" % (opt, pkg))
		_install(pkg)
	for diff in check(env, action = 'prune'):
		opt, pkg = diff
		env.log("%s %s" % (opt, pkg))
		_prune(pkg)

def _call(cmd):
	callCheck(cmd, env = _cmdenv)

def _update():
	_call(['apt-get', 'update'])

def _remove(pkg):
	_call(['apt-get', 'autoremove', '-yy', '--purge', pkg])

def _install(pkg):
	_call(['apt-get', 'install', '-yy', '--purge', '--no-install-recommends', pkg])

def _prune(pkg):
	_call(['apt-get', 'autoremove', '-yy', '--purge', pkg])
